@@ -150,6 +150,10 @@ static TagInfo
     {  0xA40B, (char *) "DeviceSettingDescription", (char *) "Device settings description"},
     {  0xA40C, (char *) "SubjectDistanceRange", (char *) "Subject distance range"},
     {  0xA420, (char *) "ImageUniqueID", (char *) "Unique image ID"},
+	// These tags are not part of the Exiv v2.2 specifications but are often loaded by applications as Exif data
+	{  0x013C, (char *) "HostComputer", (char *) "Host computer used to generate the image"},
+	{  0x4746, (char *) "Rating", (char *) "Rating tag used by Windows"},
+	{  0x4749, (char *) "RatingPercent", (char *) "Rating tag used by Windows, value in percent"},
     {  0x0000, (char *) NULL, (char *) NULL}
   };
 
@@ -228,78 +232,178 @@ static TagInfo
     {  0x0007, (char *) "CanonFirmwareVersion", (char *) NULL},
     {  0x0008, (char *) "FileNumber", (char *) NULL},
     {  0x0009, (char *) "OwnerName", (char *) NULL},
-    {  0x000A, (char *) "ColorInfoD30", (char *) "Canon ColorInfoD30 Tags"},
+    {  0x000A, (char *) "UnknownD30", (char *) "Canon UnknownD30 Tags"},
     {  0x000C, (char *) "SerialNumber", (char *) NULL},
     {  0x000D, (char *) "CanonCameraInfo", (char *) "Canon CameraInfo Tags"},
     {  0x000E, (char *) "CanonFileLength", (char *) NULL},
     {  0x000F, (char *) "CanonCustomFunctions", (char *) "Custom Functions"},
     {  0x0010, (char *) "CanonModelID", (char *) NULL},
-    {  0x0012, (char *) "CanonPictureInfo", (char *) "Canon PictureInfo Tags"},
+    {  0x0012, (char *) "CanonAFInfo", (char *) "Canon AFInfo Tags"},
+    {  0x0013, (char *) "ThumbnailImageValidArea", (char *) NULL},
     {  0x0015, (char *) "SerialNumberFormat", (char *) NULL},
+    {  0x001A, (char *) "SuperMacro", (char *) NULL},
+    {  0x001C, (char *) "DateStampMode", (char *) NULL},
+    {  0x001D, (char *) "MyColors", (char *) NULL},
+    {  0x001E, (char *) "FirmwareRevision", (char *) NULL},
+    {  0x0023, (char *) "Categories", (char *) NULL},
+    {  0x0024, (char *) "FaceDetect1", (char *) NULL},
+    {  0x0025, (char *) "FaceDetect2", (char *) NULL},
+    {  0x0026, (char *) "CanonAFInfo2", (char *) "Canon AFInfo2 Tags"},
+    {  0x0028, (char *) "ImageUniqueID", (char *) NULL},
+    {  0x0081, (char *) "RawDataOffset", (char *) NULL},
+    {  0x0083, (char *) "OriginalDecisionDataOffset", (char *) NULL},
+    {  0x0090, (char *) "CustomFunctions1D", (char *) "CanonCustom Functions1D Tags"},
+    {  0x0091, (char *) "PersonalFunctions", (char *) "CanonCustom PersonalFuncs Tags"},
+    {  0x0092, (char *) "PersonalFunctionValues", (char *) "CanonCustom PersonalFuncValues Tags"},
+    {  0x0093, (char *) "CanonFileInfo", (char *) "Canon FileInfo Tags"},
+    {  0x0094, (char *) "AFPointsInFocus1D", (char *) NULL},
+    {  0x0095, (char *) "LensModel", (char *) NULL},
+    {  0x0096, (char *) "SerialInfo", (char *) NULL},
+    {  0x0097, (char *) "DustRemovalData", (char *) NULL},
+    {  0x0099, (char *) "CustomFunctions2", (char *) NULL},
+    {  0x00A0, (char *) "ProcessingInfo", (char *) NULL},
+    {  0x00A1, (char *) "ToneCurveTable", (char *) NULL},
+    {  0x00A2, (char *) "SharpnessTable", (char *) NULL},
+    {  0x00A3, (char *) "SharpnessFreqTable", (char *) NULL},
+    {  0x00A4, (char *) "WhiteBalanceTable", (char *) NULL},
+    {  0x00A9, (char *) "ColorBalance", (char *) NULL},
+    {  0x00AA, (char *) "MeasuredColor", (char *) NULL},
+    {  0x00AE, (char *) "ColorTemperature", (char *) NULL},
+    {  0x00B0, (char *) "CanonFlags", (char *) NULL},
+    {  0x00B1, (char *) "ModifiedInfo", (char *) NULL},
+    {  0x00B2, (char *) "ToneCurveMatching", (char *) NULL},
+    {  0x00B3, (char *) "WhiteBalanceMatching", (char *) NULL},
+    {  0x00B4, (char *) "ColorSpace", (char *) NULL},
+    {  0x00B6, (char *) "PreviewImageInfo", (char *) NULL},
+    {  0x00D0, (char *) "VRDOffset", (char *) "Offset of VRD 'recipe data' if it exists"},
+    {  0x00E0, (char *) "SensorInfo", (char *) NULL},
+    {  0x4001, (char *) "ColorData", (char *) NULL},
+    {  0x4002, (char *) "UnknownBlock1?", (char *) NULL},
+    {  0x4003, (char *) "ColorInfo", (char *) NULL},
+    {  0x4005, (char *) "UnknownBlock2?", (char *) NULL},
+    {  0x4008, (char *) "BlackLevel?", (char *) NULL},
+    {  0x4013, (char *) "AFMicroAdj", (char *) NULL},
+    {  0x4015, (char *) "VignettingCorr", (char *) NULL},
+    {  0x4016, (char *) "VignettingCorr2", (char *) NULL},
+    {  0x4018, (char *) "LightingOpt", (char *) NULL},
 
 	// These 'sub'-tag values have been created for consistency -- they don't exist within the exif segment
 
 	// Fields under tag 0x0001 (we add 0xC100 to make unique tag id)
-    {  0xC101, (char *) "CameraSettings:MacroMode", (char *) NULL},
-    {  0xC102, (char *) "CameraSettings:SelfTimerDelay", (char *) NULL},
-    {  0xC103, (char *) "CameraSettings:Quality", (char *) NULL},
-    {  0xC104, (char *) "CameraSettings:CanonFlashMode", (char *) NULL},
-    {  0xC105, (char *) "CameraSettings:ContinuousDrive", (char *) NULL},
-    {  0xC107, (char *) "CameraSettings:FocusMode", (char *) NULL},
-    {  0xC10A, (char *) "CameraSettings:CanonImageSize", (char *) NULL},
-    {  0xC10B, (char *) "CameraSettings:EasyMode", (char *) NULL},
-    {  0xC10C, (char *) "CameraSettings:DigitalZoom", (char *) NULL},
-    {  0xC10D, (char *) "CameraSettings:Contrast", (char *) NULL},
-    {  0xC10E, (char *) "CameraSettings:Saturation", (char *) NULL},
-    {  0xC10F, (char *) "CameraSettings:Sharpness", (char *) NULL},
-    {  0xC110, (char *) "CameraSettings:CameraISO", (char *) NULL},
-    {  0xC111, (char *) "CameraSettings:MeteringMode", (char *) NULL},
-    {  0xC112, (char *) "CameraSettings:FocusRange", (char *) NULL},
-    {  0xC113, (char *) "CameraSettings:AFPoint", (char *) NULL},
-    {  0xC114, (char *) "CameraSettings:CanonExposureMode", (char *) NULL},
-    {  0xC116, (char *) "CameraSettings:LensType", (char *) NULL},
-    {  0xC117, (char *) "CameraSettings:LongFocal", (char *) NULL},
-    {  0xC118, (char *) "CameraSettings:ShortFocal", (char *) NULL},
-    {  0xC119, (char *) "CameraSettings:FocalUnits", (char *) "Focal Units per mm"},
-    {  0xC11A, (char *) "CameraSettings:MaxAperture", (char *) NULL},
-    {  0xC11B, (char *) "CameraSettings:MinAperture", (char *) NULL},
-    {  0xC11C, (char *) "CameraSettings:FlashActivity", (char *) NULL},
-    {  0xC11D, (char *) "CameraSettings:FlashBits", (char *) NULL},
-    {  0xC120, (char *) "CameraSettings:FocusContinuous", (char *) NULL},
-    {  0xC121, (char *) "CameraSettings:AESetting", (char *) NULL},
-    {  0xC122, (char *) "CameraSettings:ImageStabilization", (char *) NULL},
-    {  0xC123, (char *) "CameraSettings:DisplayAperture", (char *) NULL},
-    {  0xC124, (char *) "CameraSettings:ZoomSourceWidth", (char *) NULL},
-    {  0xC125, (char *) "CameraSettings:ZoomTargetWidth", (char *) NULL},
-    {  0xC128, (char *) "CameraSettings:PhotoEffect", (char *) NULL},
-    {  0xC12A, (char *) "CameraSettings:ColorTone", (char *) NULL},
+    {  0xC100 + 1, (char *) "CameraSettings:MacroMode", (char *) NULL},
+    {  0xC100 + 2, (char *) "CameraSettings:SelfTimer", (char *) NULL},
+    {  0xC100 + 3, (char *) "CameraSettings:Quality", (char *) NULL},
+    {  0xC100 + 4, (char *) "CameraSettings:CanonFlashMode", (char *) NULL},
+    {  0xC100 + 5, (char *) "CameraSettings:ContinuousDrive", (char *) NULL},
+    {  0xC100 + 7, (char *) "CameraSettings:FocusMode", (char *) NULL},
+    {  0xC100 + 9, (char *) "CameraSettings:RecordMode", (char *) NULL},
+    {  0xC100 + 10, (char *) "CameraSettings:CanonImageSize", (char *) NULL},
+    {  0xC100 + 11, (char *) "CameraSettings:EasyMode", (char *) NULL},
+    {  0xC100 + 12, (char *) "CameraSettings:DigitalZoom", (char *) NULL},
+    {  0xC100 + 13, (char *) "CameraSettings:Contrast", (char *) NULL},
+    {  0xC100 + 14, (char *) "CameraSettings:Saturation", (char *) NULL},
+    {  0xC100 + 15, (char *) "CameraSettings:Sharpness", (char *) NULL},
+    {  0xC100 + 16, (char *) "CameraSettings:CameraISO", (char *) NULL},
+    {  0xC100 + 17, (char *) "CameraSettings:MeteringMode", (char *) NULL},
+    {  0xC100 + 18, (char *) "CameraSettings:FocusRange", (char *) NULL},
+    {  0xC100 + 19, (char *) "CameraSettings:AFPoint", (char *) NULL},
+    {  0xC100 + 20, (char *) "CameraSettings:CanonExposureMode", (char *) NULL},
+    {  0xC100 + 22, (char *) "CameraSettings:LensType", (char *) NULL},
+    {  0xC100 + 23, (char *) "CameraSettings:LongFocal", (char *) NULL},
+    {  0xC100 + 24, (char *) "CameraSettings:ShortFocal", (char *) NULL},
+    {  0xC100 + 25, (char *) "CameraSettings:FocalUnits", (char *) "Focal Units per mm"},
+    {  0xC100 + 26, (char *) "CameraSettings:MaxAperture", (char *) NULL},
+    {  0xC100 + 27, (char *) "CameraSettings:MinAperture", (char *) NULL},
+    {  0xC100 + 28, (char *) "CameraSettings:FlashActivity", (char *) NULL},
+    {  0xC100 + 29, (char *) "CameraSettings:FlashBits", (char *) NULL},
+    {  0xC100 + 32, (char *) "CameraSettings:FocusContinuous", (char *) NULL},
+    {  0xC100 + 33, (char *) "CameraSettings:AESetting", (char *) NULL},
+    {  0xC100 + 34, (char *) "CameraSettings:ImageStabilization", (char *) NULL},
+    {  0xC100 + 35, (char *) "CameraSettings:DisplayAperture", (char *) NULL},
+    {  0xC100 + 36, (char *) "CameraSettings:ZoomSourceWidth", (char *) NULL},
+    {  0xC100 + 37, (char *) "CameraSettings:ZoomTargetWidth", (char *) NULL},
+    {  0xC100 + 39, (char *) "CameraSettings:SpotMeteringMode", (char *) NULL},
+    {  0xC100 + 40, (char *) "CameraSettings:PhotoEffect", (char *) NULL},
+    {  0xC100 + 41, (char *) "CameraSettings:ManualFlashOutput", (char *) NULL},
+    {  0xC100 + 42, (char *) "CameraSettings:ColorTone", (char *) NULL},
+    {  0xC100 + 46, (char *) "CameraSettings:SRAWQuality", (char *) NULL},
 
 	// Fields under tag 0x0002 (we add 0xC200 to make unique tag id)
-    {  0xC200, (char *) "FocalLength:FocalType", (char *) NULL},
-    {  0xC201, (char *) "FocalLength:FocalLength", (char *) NULL},
-    {  0xC202, (char *) "FocalLength:FocalPlaneXSize", (char *) NULL},
-    {  0xC203, (char *) "FocalLength:FocalPlaneYSize", (char *) NULL},
+    {  0xC200 + 0, (char *) "FocalLength:FocalType", (char *) NULL},
+    {  0xC200 + 1, (char *) "FocalLength:FocalLength", (char *) NULL},
+    {  0xC200 + 2, (char *) "FocalLength:FocalPlaneXSize", (char *) NULL},
+    {  0xC200 + 3, (char *) "FocalLength:FocalPlaneYSize", (char *) NULL},
 	
 	// Fields under tag 0x0004 (we add 0xC400 to make unique tag id)
-    {  0xC402, (char *) "ShotInfo:ISO", (char *) NULL},
-    {  0xC404, (char *) "ShotInfo:TargetAperture", (char *) NULL},
-    {  0xC405, (char *) "ShotInfo:TargetExposureTime", (char *) NULL},
-    {  0xC406, (char *) "ShotInfo:ExposureCompensation", (char *) NULL},
-    {  0xC407, (char *) "ShotInfo:WhiteBalance", (char *) NULL},
-    {  0xC408, (char *) "ShotInfo:SlowShutter", (char *) NULL},
-    {  0xC409, (char *) "ShotInfo:SequenceNumber", (char *) NULL},
-    {  0xC40D, (char *) "ShotInfo:FlashGuideNumber", (char *) NULL},
-    {  0xC40E, (char *) "ShotInfo:AFPointUsed", (char *) NULL},
-    {  0xC40F, (char *) "ShotInfo:FlashExposureComp", (char *) NULL},
-    {  0xC410, (char *) "ShotInfo:AutoExposureBracketing", (char *) NULL},
-    {  0xC411, (char *) "ShotInfo:AEBBracketValue", (char *) NULL},
-    {  0xC413, (char *) "ShotInfo:FocusDistanceUpper", (char *) NULL},
-    {  0xC414, (char *) "ShotInfo:FocusDistanceLower", (char *) NULL},
-    {  0xC415, (char *) "ShotInfo:FNumber", (char *) NULL},
-    {  0xC416, (char *) "ShotInfo:ExposureTime", (char *) NULL},
-    {  0xC418, (char *) "ShotInfo:BulbDuration", (char *) NULL},
-    {  0xC41B, (char *) "ShotInfo:AutoRotate", (char *) NULL},
-    {  0xC41D, (char *) "ShotInfo:SelfTimer2", (char *) NULL},
+    {  0xC400 + 1, (char *) "ShotInfo:AutoISO", (char *) NULL},
+	{  0xC400 + 2, (char *) "ShotInfo:BaseISO", (char *) NULL},
+	{  0xC400 + 3, (char *) "ShotInfo:MeasuredEV", (char *) NULL},
+    {  0xC400 + 4, (char *) "ShotInfo:TargetAperture", (char *) NULL},
+    {  0xC400 + 5, (char *) "ShotInfo:TargetExposureTime", (char *) NULL},
+    {  0xC400 + 6, (char *) "ShotInfo:ExposureCompensation", (char *) NULL},
+    {  0xC400 + 7, (char *) "ShotInfo:WhiteBalance", (char *) NULL},
+    {  0xC400 + 8, (char *) "ShotInfo:SlowShutter", (char *) NULL},
+    {  0xC400 + 9, (char *) "ShotInfo:SequenceNumber", (char *) NULL},
+    {  0xC400 + 10, (char *) "ShotInfo:OpticalZoomCode", (char *) NULL},
+    {  0xC400 + 13, (char *) "ShotInfo:FlashGuideNumber", (char *) NULL},
+    {  0xC400 + 14, (char *) "ShotInfo:AFPointsInFocus", (char *) NULL},
+    {  0xC400 + 15, (char *) "ShotInfo:FlashExposureComp", (char *) NULL},
+    {  0xC400 + 16, (char *) "ShotInfo:AutoExposureBracketing", (char *) NULL},
+    {  0xC400 + 17, (char *) "ShotInfo:AEBBracketValue", (char *) NULL},
+    {  0xC400 + 18, (char *) "ShotInfo:ControlMode", (char *) NULL},
+    {  0xC400 + 19, (char *) "ShotInfo:FocusDistanceUpper", (char *) NULL},
+    {  0xC400 + 20, (char *) "ShotInfo:FocusDistanceLower", (char *) NULL},
+    {  0xC400 + 21, (char *) "ShotInfo:FNumber", (char *) NULL},
+    {  0xC400 + 22, (char *) "ShotInfo:ExposureTime", (char *) NULL},
+    {  0xC400 + 23, (char *) "ShotInfo:MeasuredEV2", (char *) NULL},
+    {  0xC400 + 24, (char *) "ShotInfo:BulbDuration", (char *) NULL},
+    {  0xC400 + 26, (char *) "ShotInfo:CameraType", (char *) NULL},
+    {  0xC400 + 27, (char *) "ShotInfo:AutoRotate", (char *) NULL},
+    {  0xC400 + 28, (char *) "ShotInfo:NDFilter", (char *) NULL},
+    {  0xC400 + 29, (char *) "ShotInfo:SelfTimer2", (char *) NULL},
+    {  0xC400 + 33, (char *) "ShotInfo:FlashOutput", (char *) NULL},
+
+	// Fields under tag 0x0012 (we add 0xC120 to make unique tag id)
+    {  0xC120 + 0, (char *) "AFInfo:NumAFPoints", (char *) NULL},
+    {  0xC120 + 1, (char *) "AFInfo:ValidAFPoints", (char *) NULL},
+    {  0xC120 + 2, (char *) "AFInfo:CanonImageWidth", (char *) NULL},
+    {  0xC120 + 3, (char *) "AFInfo:CanonImageHeight", (char *) NULL},
+    {  0xC120 + 4, (char *) "AFInfo:AFImageWidth", (char *) NULL},
+    {  0xC120 + 5, (char *) "AFInfo:AFImageHeight", (char *) NULL},
+    {  0xC120 + 6, (char *) "AFInfo:AFAreaWidth", (char *) NULL},
+    {  0xC120 + 7, (char *) "AFInfo:AFAreaHeight", (char *) NULL},
+    {  0xC120 + 8, (char *) "AFInfo:AFAreaXPositions", (char *) NULL},
+    {  0xC120 + 9, (char *) "AFInfo:AFAreaYPositions", (char *) NULL},
+    {  0xC120 + 10, (char *) "AFInfo:AFPointsInFocus", (char *) NULL},
+    {  0xC120 + 11, (char *) "AFInfo:PrimaryAFPoint?", (char *) NULL},
+    {  0xC120 + 12, (char *) "AFInfo:PrimaryAFPoint", (char *) NULL},
+
+	// Fields under tag 0x00A0 (we add 0xCA00 to make unique tag id)
+    {  0xCA00 + 1, (char *) "ProcessingInfo:ToneCurve", (char *) NULL},
+    {  0xCA00 + 2, (char *) "ProcessingInfo:Sharpness", (char *) NULL},
+    {  0xCA00 + 3, (char *) "ProcessingInfo:SharpnessFrequency", (char *) NULL},
+    {  0xCA00 + 4, (char *) "ProcessingInfo:SensorRedLevel", (char *) NULL},
+    {  0xCA00 + 5, (char *) "ProcessingInfo:SensorBlueLevel", (char *) NULL},
+    {  0xCA00 + 6, (char *) "ProcessingInfo:WhiteBalanceRed", (char *) NULL},
+    {  0xCA00 + 7, (char *) "ProcessingInfo:WhiteBalanceBlue", (char *) NULL},
+    {  0xCA00 + 8, (char *) "ProcessingInfo:WhiteBalance", (char *) NULL},
+    {  0xCA00 + 9, (char *) "ProcessingInfo:ColorTemperature", (char *) NULL},
+    {  0xCA00 + 10, (char *) "ProcessingInfo:PictureStyle", (char *) NULL},
+    {  0xCA00 + 11, (char *) "ProcessingInfo:DigitalGain", (char *) NULL},
+    {  0xCA00 + 12, (char *) "ProcessingInfo:WBShiftAB", (char *) NULL},
+    {  0xCA00 + 13, (char *) "ProcessingInfo:WBShiftGM", (char *) NULL},
+
+	// Fields under tag 0x00E0 (we add 0xCE00 to make unique tag id)
+    {  0xCE00 + 1, (char *) "SensorInfo:SensorWidth", (char *) NULL},
+    {  0xCE00 + 2, (char *) "SensorInfo:SensorHeight", (char *) NULL},
+    {  0xCE00 + 5, (char *) "SensorInfo:SensorLeftBorder", (char *) NULL},
+    {  0xCE00 + 6, (char *) "SensorInfo:SensorTopBorder", (char *) NULL},
+    {  0xCE00 + 7, (char *) "SensorInfo:SensorRightBorder", (char *) NULL},
+    {  0xCE00 + 8, (char *) "SensorInfo:SensorBottomBorder", (char *) NULL},
+    {  0xCE00 + 9, (char *) "SensorInfo:BlackMaskLeftBorder", (char *) NULL},
+    {  0xCE00 + 10, (char *) "SensorInfo:BlackMaskTopBorder", (char *) NULL},
+    {  0xCE00 + 11, (char *) "SensorInfo:BlackMaskRightBorder", (char *) NULL},
+    {  0xCE00 + 12, (char *) "SensorInfo:BlackMaskBottomBorder", (char *) NULL},
 
     {  0x0000, (char *) NULL, (char *) NULL}
   };
@@ -347,8 +451,10 @@ static TagInfo
     {  0x0021, (char *) "Sharpness", (char *) NULL},
     {  0x0E00, (char *) "PrintIM", (char *) NULL},
     {  0x2000, (char *) "PreviewImage", (char *) NULL},
+    {  0x2001, (char *) "FirmwareDate", (char *) NULL},
     {  0x2011, (char *) "WhiteBalanceBias", (char *) NULL},
     {  0x2012, (char *) "WhiteBalance", (char *) NULL},
+    {  0x2021, (char *) "AFPointPosition", (char *) NULL},
     {  0x2022, (char *) "ObjectDistance", (char *) NULL},
     {  0x2034, (char *) "FlashDistance", (char *) NULL},
     {  0x3000, (char *) "RecordMode", (char *) NULL},
@@ -368,6 +474,10 @@ static TagInfo
     {  0x301C, (char *) "SequenceNumber", (char *) NULL},
     {  0x301D, (char *) "BracketSequence", (char *) NULL},
     {  0x3020, (char *) "ImageStabilization", (char *) NULL},
+    {  0x302A, (char *) "LightingMode", (char *) NULL},
+    {  0x302B, (char *) "PortraitRefiner", (char *) NULL},
+    {  0x3030, (char *) "SpecialEffectLevel", (char *) NULL},
+    {  0x3031, (char *) "SpecialEffectSetting", (char *) NULL},
     {  0x0000, (char *) NULL, (char *) NULL}
   };
 
@@ -378,6 +488,7 @@ static TagInfo
   exif_fujifilm_tag_table[] =
   {
     {  0x0000, (char *) "MakernoteVersion", (char *) NULL},
+    {  0x0010, (char *) "InternalSerialNumber", (char *) NULL},
     {  0x1000, (char *) "Quality", (char *) NULL},
     {  0x1001, (char *) "Sharpness", (char *) NULL},
     {  0x1002, (char *) "WhiteBalance", (char *) NULL},
@@ -387,12 +498,16 @@ static TagInfo
     {  0x1011, (char *) "FlashStrength", (char *) NULL},
     {  0x1020, (char *) "Macro", (char *) NULL},
     {  0x1021, (char *) "FocusMode", (char *) NULL},
+    {  0x1023, (char *) "FocusPixel", (char *) NULL},
     {  0x1030, (char *) "SlowSynchro", (char *) NULL},
     {  0x1031, (char *) "PictureMode", (char *) NULL},
     {  0x1100, (char *) "AutoBracketting", (char *) NULL},
+    {  0x1101, (char *) "SequenceNumber", (char *) NULL},
+    {  0x1210, (char *) "ColorMode", (char *) NULL},
     {  0x1300, (char *) "BlurWarning", (char *) NULL},
     {  0x1301, (char *) "FocusWarning", (char *) NULL},
     {  0x1302, (char *) "ExposureWarning", (char *) NULL},
+    {  0x1400, (char *) "DynamicRange", (char *) NULL},
     {  0x0000, (char *) NULL, (char *) NULL}
   };
 
@@ -408,10 +523,10 @@ static TagInfo
   };
 
 /**
-Olympus / Epson / Agfa maker note
+Olympus Type 1 / Epson / Agfa maker note
 */
 static TagInfo
-  exif_olympus_tag_table[] =
+  exif_olympus_type1__tag_table[] =
   {
     {  0x0000, (char *) "MakerNoteVersion", (char *) NULL},
     {  0x0001, (char *) "MinoltaCameraSettingsOld", (char *) NULL},
@@ -432,6 +547,10 @@ static TagInfo
     {  0x0207, (char *) "CameraType", (char *) NULL},
     {  0x0208, (char *) "TextInfo", (char *) "Olympus TextInfo Tags"},
     {  0x0209, (char *) "CameraID", (char *) NULL},
+    {  0x020B, (char *) "EpsonImageWidth", (char *) NULL},
+    {  0x020C, (char *) "EpsonImageHeight", (char *) NULL},
+    {  0x020D, (char *) "EpsonSoftware", (char *) NULL},
+    {  0x0280, (char *) "PreviewImage", (char *) NULL},
     {  0x0300, (char *) "PreCaptureFrames", (char *) NULL},
     {  0x0302, (char *) "OneTouchWB", (char *) NULL}, 
     {  0x0404, (char *) "SerialNumber", (char *) NULL}, 
@@ -493,14 +612,33 @@ Minolta maker note
 static TagInfo
   exif_minolta_tag_table[] =
   {
-    {  0x0000, (char *) "Version", (char *) NULL},
+    {  0x0000, (char *) "MakerNoteVersion", (char *) NULL},
+    {  0x0001, (char *) "MinoltaCameraSettingsOld", (char *) NULL},
+    {  0x0003, (char *) "MinoltaCameraSettings", (char *) NULL},
+    {  0x0004, (char *) "MinoltaCameraSettings7D", (char *) NULL},
+    {  0x0018, (char *) "ImageStabilization", (char *) NULL},
 	{  0x0040, (char *) "CompressedImageSize", (char *) NULL},
-    {  0x0081, (char *) "JPEGThumbnailImage", (char *) NULL},
-    {  0x0088, (char *) "JPEGThumbnailOffset", (char *) NULL},
-    {  0x0089, (char *) "JPEGThumbnailLength", (char *) NULL},
+    {  0x0081, (char *) "PreviewImage", (char *) NULL},
+    {  0x0088, (char *) "PreviewImageStart", (char *) NULL},
+    {  0x0089, (char *) "PreviewImageLength", (char *) NULL},
+    {  0x0100, (char *) "SceneMode", (char *) NULL},
     {  0x0101, (char *) "ColorMode", (char *) NULL},
-    {  0x0102, (char *) "ImageQuality", (char *) NULL},
-    {  0x0103, (char *) "ImageQuality2", (char *) NULL},
+    {  0x0102, (char *) "MinoltaQuality", (char *) NULL},
+    {  0x0103, (char *) "MinoltaImageSize", (char *) NULL},
+    {  0x0104, (char *) "FlashExposureComp", (char *) NULL},
+    {  0x0105, (char *) "Teleconverter", (char *) NULL},
+    {  0x0107, (char *) "ImageStabilization", (char *) NULL},
+    {  0x0109, (char *) "RawAndJpgRecording", (char *) NULL},
+    {  0x010A, (char *) "ZoneMatching", (char *) NULL},
+    {  0x010B, (char *) "ColorTemperature", (char *) NULL},
+    {  0x010C, (char *) "LensType", (char *) NULL},
+    {  0x0111, (char *) "ColorCompensationFilter", (char *) NULL},
+    {  0x0112, (char *) "WhiteBalanceFineTune", (char *) NULL},
+    {  0x0113, (char *) "ImageStabilization", (char *) NULL},
+    {  0x0114, (char *) "MinoltaCameraSettings5D", (char *) NULL},
+    {  0x0115, (char *) "WhiteBalance", (char *) NULL},
+    {  0x0E00, (char *) "PrintIM", (char *) NULL},
+    {  0x0F00, (char *) "MinoltaCameraSettings2", (char *) NULL},
     {  0x0000, (char *) NULL, (char *) NULL}
   };
 
@@ -577,7 +715,7 @@ static TagInfo
     {  0x000D, (char *) "ProgramShift", (char *) NULL},
     {  0x000E, (char *) "ExposureDifference", (char *) NULL},
     {  0x000F, (char *) "ISOSelection", (char *) NULL},
-    {  0x0011, (char *) "ThumbnailIFDOffset", (char *) NULL},
+    {  0x0011, (char *) "NikonPreview", (char *) NULL},
     {  0x0012, (char *) "FlashExposureComp", (char *) NULL},
     {  0x0013, (char *) "ISOSetting", (char *) NULL},
     {  0x0016, (char *) "ImageBoundary", (char *) NULL},
@@ -609,6 +747,8 @@ static TagInfo
     {  0x0096, (char *) "NEFCurve2", (char *) NULL},
     {  0x0099, (char *) "RawImageCenter", (char *) NULL},
     {  0x009A, (char *) "SensorPixelSize", (char *) NULL},
+    {  0x009C, (char *) "SceneAssist", (char *) NULL},
+    {  0x009E, (char *) "RetouchHistory", (char *) NULL},
     {  0x00A0, (char *) "SerialNumber2", (char *) NULL},
     {  0x00A2, (char *) "ImageDataSize", (char *) NULL},
     {  0x00A5, (char *) "ImageCount", (char *) NULL},
@@ -619,11 +759,21 @@ static TagInfo
     {  0x00AB, (char *) "VariProgram", (char *) NULL},
     {  0x00AC, (char *) "ImageStabilization", (char *) NULL},
     {  0x00AD, (char *) "AFResponse", (char *) NULL},
+    {  0x00B0, (char *) "MultiExposure", (char *) NULL},
     {  0x00B1, (char *) "HighISONoiseReduction", (char *) NULL},
+    {  0x00B3, (char *) "ToningEffect", (char *) NULL},
+    {  0x00B6, (char *) "PowerUpTime", (char *) NULL},
+    {  0x00B7, (char *) "AFInfo2", (char *) NULL},
+    {  0x00B8, (char *) "FileInfo", (char *) NULL},
+    {  0x00B9, (char *) "AFTune", (char *) NULL},
+    {  0x00BD, (char *) "PictureControl", (char *) NULL},
     {  0x0E00, (char *) "PrintIM", (char *) NULL},
     {  0x0E01, (char *) "NikonCaptureData", (char *) NULL},
     {  0x0E09, (char *) "NikonCaptureVersion", (char *) NULL},
     {  0x0E0E, (char *) "NikonCaptureOffsets", (char *) NULL},
+    {  0x0E10, (char *) "NikonScanIFD", (char *) NULL},
+    {  0x0E1D, (char *) "NikonICCProfile", (char *) NULL},
+    {  0x0E1E, (char *) "NikonCaptureOutput", (char *) NULL},
     {  0x0000, (char *) NULL, (char *) NULL}
   };
 
@@ -684,6 +834,7 @@ static TagInfo
 static TagInfo
   exif_pentax_tag_table[] =
   {
+    {  0x0000, (char *) "PentaxVersion", (char *) NULL},
     {  0x0001, (char *) "PentaxMode", (char *) NULL},
     {  0x0002, (char *) "PreviewImageSize", (char *) NULL},
     {  0x0003, (char *) "PreviewImageLength", (char *) NULL},
@@ -696,13 +847,15 @@ static TagInfo
     {  0x000B, (char *) "PictureMode", (char *) NULL},
     {  0x000C, (char *) "FlashMode", (char *) NULL},
     {  0x000D, (char *) "FocusMode", (char *) NULL},
-    {  0x000F, (char *) "AutoAFPoint", (char *) NULL},
+    {  0x000E, (char *) "AFPointSelected", (char *) NULL},
+    {  0x000F, (char *) "AFPointsInFocus", (char *) NULL},
     {  0x0010, (char *) "FocusPosition", (char *) NULL},
     {  0x0012, (char *) "ExposureTime", (char *) NULL},
     {  0x0013, (char *) "FNumber", (char *) NULL},
     {  0x0014, (char *) "ISO", (char *) NULL},
     {  0x0016, (char *) "ExposureCompensation", (char *) NULL},
     {  0x0017, (char *) "MeteringMode", (char *) NULL},
+    {  0x0018, (char *) "AutoBracketing", (char *) NULL},
     {  0x0019, (char *) "WhiteBalance", (char *) NULL},
     {  0x001A, (char *) "WhiteBalanceMode", (char *) NULL},
     {  0x001B, (char *) "BlueBalance", (char *) NULL},
@@ -717,14 +870,56 @@ static TagInfo
     {  0x0024, (char *) "DestinationCity", (char *) NULL},
     {  0x0025, (char *) "HometownDST", (char *) NULL},
     {  0x0026, (char *) "DestinationDST", (char *) NULL},
+    {  0x0027, (char *) "DSPFirmwareVersion", (char *) NULL},
+    {  0x0028, (char *) "CPUFirmwareVersion", (char *) NULL},
     {  0x0029, (char *) "FrameNumber", (char *) NULL},
+    {  0x002D, (char *) "EffectiveLV", (char *) NULL},
+    {  0x0032, (char *) "ImageProcessing", (char *) NULL},
+    {  0x0033, (char *) "PictureMode", (char *) NULL},
+    {  0x0034, (char *) "DriveMode", (char *) NULL},
     {  0x0037, (char *) "ColorSpace", (char *) NULL},
     {  0x0039, (char *) "RawImageSize", (char *) NULL},
+    {  0x003E, (char *) "PreviewImageBorders", (char *) NULL},
     {  0x003F, (char *) "LensType", (char *) NULL},
+    {  0x0040, (char *) "SensitivityAdjust", (char *) NULL},
+    {  0x0041, (char *) "ImageProcessingCount", (char *) NULL},
+    {  0x0047, (char *) "CameraTemperature", (char *) NULL},
+    {  0x0048, (char *) "AELock", (char *) NULL},
     {  0x0049, (char *) "NoiseReduction", (char *) NULL},
+    {  0x004D, (char *) "FlashExposureComp", (char *) NULL},
+    {  0x004F, (char *) "ImageTone", (char *) NULL},
+	{  0x0050, (char *) "ColorTemperature", (char *) NULL},
+    {  0x005C, (char *) "ShakeReductionInfo", (char *) NULL},
+    {  0x005D, (char *) "ShutterCount", (char *) NULL},
+    {  0x0069, (char *) "DynamicRangeExpansion", (char *) NULL},
+    {  0x0071, (char *) "HighISONoiseReduction", (char *) NULL},
+    {  0x0072, (char *) "AFAdjustment", (char *) NULL},
     {  0x0200, (char *) "BlackPoint", (char *) NULL},
     {  0x0201, (char *) "WhitePoint", (char *) NULL},
-    {  0x03FE, (char *) "DataDump", (char *) NULL},
+    {  0x0203, (char *) "ColorMatrixA", (char *) NULL},
+    {  0x0204, (char *) "ColorMatrixB", (char *) NULL},
+    {  0x0205, (char *) "CameraSettings", (char *) NULL},
+	{  0x0206, (char *) "AEInfo", (char *) NULL},
+    {  0x0207, (char *) "LensInfo", (char *) NULL},
+    {  0x0208, (char *) "FlashInfo", (char *) NULL},
+    {  0x0209, (char *) "AEMeteringSegments", (char *) NULL},
+    {  0x020A, (char *) "FlashMeteringSegments", (char *) NULL},
+    {  0x020B, (char *) "SlaveFlashMeteringSegments", (char *) NULL},
+    {  0x020D, (char *) "WB_RGGBLevelsDaylight", (char *) NULL},
+    {  0x020E, (char *) "WB_RGGBLevelsShade", (char *) NULL},
+    {  0x020F, (char *) "WB_RGGBLevelsCloudy", (char *) NULL},
+    {  0x0210, (char *) "WB_RGGBLevelsTungsten", (char *) NULL},
+    {  0x0211, (char *) "WB_RGGBLevelsFluorescentD", (char *) NULL},
+    {  0x0212, (char *) "WB_RGGBLevelsFluorescentN", (char *) NULL},
+    {  0x0213, (char *) "WB_RGGBLevelsFluorescentW", (char *) NULL},
+    {  0x0214, (char *) "WB_RGGBLevelsFlash", (char *) NULL},
+    {  0x0215, (char *) "CameraInfo", (char *) NULL},
+    {  0x0216, (char *) "BatteryInfo", (char *) NULL},
+    {  0x021B, (char *) "SaturationInfo", (char *) NULL},
+    {  0x021F, (char *) "AFInfo", (char *) NULL},
+    {  0x0222, (char *) "ColorInfo", (char *) NULL},
+    {  0x0224, (char *) "EVStepInfo", (char *) NULL},
+	{  0x03FE, (char *) "DataDump", (char *) NULL},
     {  0x0402, (char *) "ToneCurve", (char *) NULL},
     {  0x0403, (char *) "ToneCurves", (char *) NULL},
     {  0x0E00, (char *) "PrintIM", (char *) NULL},
@@ -751,32 +946,79 @@ static TagInfo
 static TagInfo
   iptc_tag_table[] =
   {
-    {  0x0200, (char *) "DirectoryVersion", (char *) "Directory Version"},
-    {  0x0278, (char *) "Caption/Abstract", (char *) "Caption"},
-    {  0x027A, (char *) "Writer/Editor", (char *) "Caption Writer"},
-    {  0x0269, (char *) "Headline", (char *) "Headline"},
-    {  0x0228, (char *) "SpecialInstructions", (char *) "Instructions"},
-    {  0x0250, (char *) "ByLine", (char *) "Author"},
-    {  0x0255, (char *) "BylineTitle", (char *) "Author's Position"},
-    {  0x026E, (char *) "Credit", (char *) "Credit"},
-    {  0x0273, (char *) "Source", (char *) "Source"},
-    {  0x0205, (char *) "ObjectName", (char *) "Title"},
-    {  0x0237, (char *) "DateCreated", (char *) "Date Created"},
-    {  0x025A, (char *) "City", (char *) "City"},
-	{  0x025C, (char *) "Sublocation", (char*) "Sub Location"}, 
-    {  0x025F, (char *) "Province/State", (char *) "State/Province"},
-    {  0x0264, (char *) "Country/PrimaryLocationCode", (char *) "Country Code"},
-	{  0x0265, (char *) "Country/PrimaryLocationName", (char *) "Country Name"},
-    {  0x0267, (char *) "OriginalTransmissionReference", (char *) "Transmission Reference"},
-    {  0x020F, (char *) "Category", (char *) "Category"},
-    {  0x0214, (char *) "SupplementalCategory", (char *) "Supplemental Categories"},
-    {  0x020A, (char *) "Urgency", (char *) "Urgency"},
-    {  0x0219, (char *) "Keywords", (char *) "Keywords"},
-    {  0x0274, (char *) "CopyrightNotice", (char *) "Copyright Notice"},
-    {  0x021E, (char *) "ReleaseDate", (char *) "Release Date"},
-    {  0x0223, (char *) "ReleaseTime", (char *) "Release Time"},
-    {  0x023C, (char *) "TimeCreated", (char *) "Time Created"},
-    {  0x0241, (char *) "OriginatingProgram", (char *) "Originating Program"},
+	  // IPTC-NAA IIM version 4
+    {  0x0200 +   0, (char *) "ApplicationRecordVersion", (char *) "Application Record Version"},
+    {  0x0200 +   3, (char *) "ObjectTypeReference", (char *) "Object Type Reference"},
+    {  0x0200 +   4, (char *) "ObjectAttributeReference", (char *) "Object Attribute Reference"},
+    {  0x0200 +   5, (char *) "ObjectName", (char *) "Title"},
+    {  0x0200 +   7, (char *) "EditStatus", (char *) "Edit Status"},
+    {  0x0200 +   8, (char *) "EditorialUpdate", (char *) "Editorial Update"},
+    {  0x0200 +  10, (char *) "Urgency", (char *) "Urgency"},
+    {  0x0200 +  12, (char *) "SubjectReference", (char *) "Subject Reference"},
+    {  0x0200 +  15, (char *) "Category", (char *) "Category"},
+    {  0x0200 +  20, (char *) "SupplementalCategories", (char *) "Supplemental Categories"},
+    {  0x0200 +  22, (char *) "FixtureIdentifier", (char *) "Fixture Identifier"},
+    {  0x0200 +  25, (char *) "Keywords", (char *) "Keywords"},
+    {  0x0200 +  26, (char *) "ContentLocationCode", (char *) "Content Location Code"},
+    {  0x0200 +  27, (char *) "ContentLocationName", (char *) "Content Location Name"},
+    {  0x0200 +  30, (char *) "ReleaseDate", (char *) "Release Date"},
+    {  0x0200 +  35, (char *) "ReleaseTime", (char *) "Release Time"},
+    {  0x0200 +  37, (char *) "ExpirationDate", (char *) "Expiration Date"},
+    {  0x0200 +  38, (char *) "ExpirationTime", (char *) "Expiration Time"},
+    {  0x0200 +  40, (char *) "SpecialInstructions", (char *) "Instructions"},
+    {  0x0200 +  42, (char *) "ActionAdvised", (char *) "Action Advised"},
+    {  0x0200 +  45, (char *) "ReferenceService", (char *) "Reference Service"},
+    {  0x0200 +  47, (char *) "ReferenceDate", (char *) "Reference Date"},
+    {  0x0200 +  50, (char *) "ReferenceNumber", (char *) "Reference Number"},
+    {  0x0200 +  55, (char *) "DateCreated", (char *) "Date Created"},
+    {  0x0200 +  60, (char *) "TimeCreated", (char *) "Time Created"},
+    {  0x0200 +  62, (char *) "DigitalCreationDate", (char *) "Digital Creation Date"},
+    {  0x0200 +  63, (char *) "DigitalCreationTime", (char *) "Digital Creation Time"},
+    {  0x0200 +  65, (char *) "OriginatingProgram", (char *) "Originating Program"},
+    {  0x0200 +  70, (char *) "ProgramVersion", (char *) "Program Version"},
+    {  0x0200 +  75, (char *) "ObjectCycle", (char *) "Object Cycle"},
+    {  0x0200 +  80, (char *) "By-line", (char *) "Author"},
+    {  0x0200 +  85, (char *) "By-lineTitle", (char *) "Author's Position"},
+    {  0x0200 +  90, (char *) "City", (char *) "City"},
+    {  0x0200 +  92, (char *) "SubLocation", (char *) "Sub-Location"},
+    {  0x0200 +  95, (char *) "Province-State", (char *) "State/Province"},
+    {  0x0200 + 100, (char *) "Country-PrimaryLocationCode", (char *) "Country Code"},
+    {  0x0200 + 101, (char *) "Country-PrimaryLocationName", (char *) "Country Name"},
+    {  0x0200 + 103, (char *) "OriginalTransmissionReference", (char *) "Transmission Reference"},
+    {  0x0200 + 105, (char *) "Headline", (char *) "Headline"},
+    {  0x0200 + 110, (char *) "Credit", (char *) "Credit"},
+    {  0x0200 + 115, (char *) "Source", (char *) "Source"},
+    {  0x0200 + 116, (char *) "CopyrightNotice", (char *) "Copyright Notice"},
+    {  0x0200 + 118, (char *) "Contact", (char *) "Contact"},
+    {  0x0200 + 120, (char *) "Caption-Abstract", (char *) "Caption"},
+    {  0x0200 + 122, (char *) "Writer-Editor", (char *) "Caption Writer"},
+    {  0x0200 + 125, (char *) "RasterizedCaption", (char *) "Rasterized Caption"},
+    {  0x0200 + 130, (char *) "ImageType", (char *) "Image Type"},
+    {  0x0200 + 131, (char *) "ImageOrientation", (char *) "Image Orientation"},
+    {  0x0200 + 135, (char *) "LanguageIdentifier", (char *) "Language Identifier"},
+    {  0x0200 + 150, (char *) "AudioType", (char *) "Audio Type"},
+    {  0x0200 + 151, (char *) "AudioSamplingRate", (char *) "Audio Sampling Rate"},
+    {  0x0200 + 152, (char *) "AudioSamplingResolution", (char *) "Audio Sampling Resolution"},
+    {  0x0200 + 153, (char *) "AudioDuration", (char *) "Audio Duration"},
+    {  0x0200 + 154, (char *) "AudioOutcue", (char *) "Audio Outcue"},
+		// Metadata seen in other softwares (see also http://owl.phy.queensu.ca/~phil/exiftool/TagNames/IPTC.html#ApplicationRecord)
+    {  0x0200 + 184, (char *) "JobID", (char *) "Job ID"},
+    {  0x0200 + 185, (char *) "MasterDocumentID", (char *) "Master Document ID"},
+    {  0x0200 + 186, (char *) "ShortDocumentID", (char *) "Short Document ID"},
+    {  0x0200 + 187, (char *) "UniqueDocumentID", (char *) "Unique Document ID"},
+    {  0x0200 + 188, (char *) "OwnerID", (char *) "Owner ID"},
+		// IPTC-NAA IIM version 4
+    {  0x0200 + 200, (char *) "ObjectPreviewFileFormat", (char *) "Object Preview File Format"},
+    {  0x0200 + 201, (char *) "ObjectPreviewFileVersion", (char *) "Object Preview File Version"},
+    {  0x0200 + 202, (char *) "ObjectPreviewData", (char *) "Audio Outcue"},
+		// Metadata seen in other softwares (see also http://owl.phy.queensu.ca/~phil/exiftool/TagNames/IPTC.html#ApplicationRecord)
+    {  0x0200 + 221, (char *) "Prefs", (char *) "PhotoMechanic preferences"},
+    {  0x0200 + 225, (char *) "ClassifyState", (char *) "Classify State"},
+    {  0x0200 + 228, (char *) "SimilarityIndex", (char *) "Similarity Index"},
+    {  0x0200 + 230, (char *) "DocumentNotes", (char *) "Document Notes"},
+    {  0x0200 + 231, (char *) "DocumentHistory", (char *) "Document History"},
+    {  0x0200 + 232, (char *) "ExifCameraInfo", (char *) "Exif Camera Info"},
+
     {  0x0000, (char *) NULL, (char *) NULL}
   };
 
@@ -846,7 +1088,7 @@ TagLib::TagLib() {
 	addMetadataModel(TagLib::EXIF_MAKERNOTE_NIKONTYPE1, exif_nikon_type1_tag_table);
 	addMetadataModel(TagLib::EXIF_MAKERNOTE_NIKONTYPE2, exif_nikon_type2_tag_table);
 	addMetadataModel(TagLib::EXIF_MAKERNOTE_NIKONTYPE3, exif_nikon_type3_tag_table);
-	addMetadataModel(TagLib::EXIF_MAKERNOTE_OLYMPUS, exif_olympus_tag_table);
+	addMetadataModel(TagLib::EXIF_MAKERNOTE_OLYMPUSTYPE1, exif_olympus_type1__tag_table);
 	addMetadataModel(TagLib::EXIF_MAKERNOTE_PANASONIC, exif_panasonic_tag_table);
 	addMetadataModel(TagLib::EXIF_MAKERNOTE_ASAHI, exif_asahi_tag_table);
 	addMetadataModel(TagLib::EXIF_MAKERNOTE_PENTAX, exif_pentax_tag_table);
@@ -975,7 +1217,7 @@ TagLib::getFreeImageModel(MDMODEL model) {
 		case EXIF_MAKERNOTE_NIKONTYPE1:
 		case EXIF_MAKERNOTE_NIKONTYPE2:
 		case EXIF_MAKERNOTE_NIKONTYPE3:
-		case EXIF_MAKERNOTE_OLYMPUS:
+		case EXIF_MAKERNOTE_OLYMPUSTYPE1:
 		case EXIF_MAKERNOTE_PANASONIC:
 		case EXIF_MAKERNOTE_ASAHI:
 		case EXIF_MAKERNOTE_PENTAX:
